﻿---------------------------------------------------------------------------------
--
-- Prat - A framework for World of Warcraft chat mods
--
-- Copyright (C) 2006-2007  Prat Development Team
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
-------------------------------------------------------------------------------
-- Gotta have Prat
if not Prat then return end

-- Get Utility Libraries
local util, DBG, CLR = GetPratUtils()

if not AceLibrary:HasInstance("Waterfall-1.0") then return end

local dewdrop = AceLibrary("Dewdrop-2.0")
local WATERFALL_ID = "Prat"

local function unpack_parms(t, i)
	local k,v = next(t, i)
	if not k then return end
	return k, v, unpack_parms(t, k)
end

local L = AceLibrary("AceLocale-2.2"):new("PratWaterfall")

L:RegisterTranslations("enUS", function() return {
    ["Waterfall Config"] = true,
    ["Opens the Waterfall Config window"] = true,
} end)

L:RegisterTranslations("deDE", function() return {
    ["Waterfall Config"] = "Waterfall Konfig",
    ["Opens the Waterfall Config window"] = "Öffnet das 'Waterfall' Konfigurationsfenster.",
} end)

L:RegisterTranslations("koKR", function() return {
    ["Waterfall Config"] = "Waterfall 설정",
    ["Opens the Waterfall Config window"] = "Waterfall 설정창을 엽니다",
} end)

PratWaterfall	= AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0")
PratWaterfall.W	= AceLibrary('Waterfall-1.0')

function PratWaterfall:RegisterWaterfall()
	local WaterfallOpts
	local p = Prat.db.profile

	Prat.Options.args.Waterfall = {
		name = L["Waterfall Config"],
		type = "execute",
		order = 340,
		wfHidden = true,
		desc = L["Opens the Waterfall Config window"],
		func = function() PratWaterfall.W:Open("Prat") dewdrop:Close() end,
		disabled = "~IsActive"
		}

	Prat.ConsoleOptions.args.waterfall = {
		name = L["Waterfall Config"],
		type = "execute",
		wfHidden = true,
		desc = L["Opens the Waterfall Config window"],
		func = function() W:Open("Prat") dewdrop:Close() end,
		disabled = "~IsActive"
		}

	WaterfallOpts = {
		aceOptions = Prat.ConsoleOptions,
		treeType = 'TREE',
		colorR = 0.5,
		colorG = 0.5,
		colorB = 1.0,
		treeLevels = 3,
		}

	PratWaterfall.W:Register(WATERFALL_ID, unpack_parms(WaterfallOpts))
end

function PratWaterfall:OnInitialize()
	self:RegisterEvent("Prat_Ready")
end

function PratWaterfall:Prat_Ready()
	self:RegisterWaterfall()

	self:RegisterEvent("Prat_ModuleDisabled", "ModuleUpdate")
	self:RegisterEvent("Prat_ModuleEnabled", "ModuleUpdate")
end

function PratWaterfall:ModuleUpdate()
	local W = PratWaterfall.W

	W:Refresh(WATERFALL_ID)
end

function PratWaterfall:Open()
	local W = PratWaterfall.W

	return W:Open('Prat')
end

function PratWaterfall:Close()
	local W = PratWaterfall.W

	return W:Close('Prat')
end

function PratWaterfall:Toggle()
	local W		= PratWaterfall.W
	local open	= W:IsOpen('Prat')

	if open then
		self:Close('Prat')
	else
		self:Open('Prat')
	end
end
